//
//      ERORB.java
//      ----------
//
//      Engine Room CORBA ORB
//
//      Copyright (c) Engine Room Software Ltd. 1998-2001
//      Copyright (c) Mitch Britton 1998-2001
//      All Rights Reserved
//      See the file licence.txt in the 
//      root directory of this distribution for licence details
//

package com.engroom.CORBA ;

public class ERORB extends ORBSingleton
{
    private BOA boa ;               // BOA has it's own thread
    private String host = null ;    // Host name
    private int port = 0 ;          // port 0 = any free port
    private int timeout = 0 ;       // client read timeout
    private int idle = 0 ;          // object idle timeout

    private java.util.Vector values ;
    private static boolean locked = false ;
    private static boolean firstConnect = true ;
    private String[] p_args ;
    private java.util.Properties p_props ;
    private InitialServices initialObj ;

    private boolean useipno         = false ;
    private int initialPort         = 900 ;
    private String initialHost      = "localhost" ;
    private String initialObjectKey = null ;

    private java.util.Hashtable services = new java.util.Hashtable() ;


    private static String ORBparams[] =
    {
        "ORBport",
        "ORBlog",
        "ORBuseipno",
        "ORBInitialObjectKey",
        "ORBtimeout",
        "ORBidle",
        "org.omg.CORBA.ORBInitialPort",
        "ORBInitialPort",
        "org.omg.CORBA.ORBInitialHost",
        "ORBInitialHost",
        "ORBhost"
    } ;

    public static void main( String[] args )
    {
        boolean ip = false ;

        if ( args.length > 0 ) 
        {
            if ( args[ 0 ].equals( "-ip" ) )
                ip = true ;
            else if ( args[ 0 ].equals( "-vm" ) )
            {
                System.out.println( vmVersion() ) ;
                return ;
            }
        }
        String host = getHost( ip ) ;

        if ( host == null )
            System.out.println( "Can't resolve host name" ) ;
        else
            System.out.println( "Host " + ( ip ? "IP address=" : "name=" ) + host ) ;
    }

    public ERORB()
    {
    }

    public static org.omg.CORBA.ORB init( String[] args, java.util.Properties props )
    {
        ERORB e = new ERORB() ;

        e.set_parameters( args, props ) ;

        // System.err.println( "ORB init .." ) ;

        return e ;
    }

    public static org.omg.CORBA.ORB init( java.applet.Applet app, java.util.Properties props )
    {
        ERORB e = new ERORB() ;

        e.set_parameters( app, props ) ;

        // System.err.println( "ORB applet init .." ) ;

        return e ;
    }

    public int getTimeout()
    {
        return timeout ;
    }

    public String getHost()
    {
        return host ;
    }

    public int getIdle()
    {
        return idle ;
    }

    private static String theHost;//wbruna
    public static String getHost( boolean ipNum )
    {
  if (theHost == null) {
        try
        {
            java.net.InetAddress lh = java.net.InetAddress.getLocalHost() ;

            if ( ipNum )
                return (theHost = lh.getHostAddress()) ;//return lh.getHostAddress() ;

            return (theHost = lh.getHostName()) ;//return lh.getHostName() ;
        }
        catch ( java.net.UnknownHostException e )
        {
        }

       }
        return theHost; //return null;
    }

    public BOA getBOA()
    {
        if ( boa == null )
            boa = new BOA( this ) ;

        return boa ;
    }

    public static boolean lock()
    {
        while ( lock( 10000 ) == false )
            System.err.println( "No lock after 10 seconds" ) ;

        return true ;
    }

    public static synchronized boolean lock( int timeout )
    {
        for ( ; ; )
        {
            if ( locked == false )
            {
                locked = true ;
                return locked ;
            }

            if ( timeout <= 0 )
                return locked ;

            try
            {
                Thread.currentThread().sleep( 100 ) ;
            }
            catch ( InterruptedException e )
            {
                timeout -= 100 ;
            }
        }
    }

    public static synchronized void unlock()
    {
        if ( locked )
            locked = false ;
    }

    public String object_to_string( org.omg.CORBA.Object obj )
    {
        ERObject e = ERObject.getMe( obj ) ;

        return e.toString() ;
    }

    private InitialServices getInitialObject()
    {
        if ( initialObj == null )
        {
            byte[] key = { ( byte )'I', ( byte )'N', ( byte )'I', ( byte )'T' } ;

            IOR ior = new IOR( initialHost, initialPort, "init", key ) ;

            org.omg.CORBA.portable.ObjectImpl impl ;

            if ( ior != null )
            {
                impl = ( org.omg.CORBA.portable.ObjectImpl )ior.createObject( this ) ;

                _InitialServicesStub stub = new _InitialServicesStub() ;

                stub._set_delegate( impl._get_delegate() ) ;

                initialObj = stub ;
            }
        }

        return initialObj ;
    }

    public String[] list_initial_services()
    {
        InitialServices obj = getInitialObject() ;

        String[] services ;

        if ( obj == null )
            services = new String[ 0 ] ;
        else
        {
            services = obj.list() ;
        }

        return services ;
    }

    public org.omg.CORBA.Object resolve_initial_references( String object_name )
        throws org.omg.CORBA.ORBPackage.InvalidName
    {
        InitialServices obj = getInitialObject() ;

        org.omg.CORBA.Object targ = null ;

        if ( obj != null )
        {
            targ = ( org.omg.CORBA.Object )services.get( object_name ) ;

            if ( targ == null )
            {
                targ = obj.get( object_name ) ;

                if ( targ != null )
                    services.put( object_name, targ ) ;
            }
            else
                targ = targ._duplicate() ;
        }
        return targ ;
    }

    public org.omg.CORBA.Object string_to_object( String str )
    {
        IOR ior = IOR.createIOR( str ) ;

        if ( ior != null )
            return ior.createObject( this ) ;

        return null ;
    }

    public org.omg.CORBA.NVList create_list( int count )
    {
        return new NVList( count ) ;
    }

    public org.omg.CORBA.NamedValue create_named_value( String name, org.omg.CORBA.Any value, int flags )
    {
        return new NamedValue( name, value, flags ) ;
    }

    public static int implementation( java.lang.String name, short port )
    {
        return 0 ;
    }


    // additional methods for IDL/Java mapping

    public org.omg.CORBA.TypeCode get_primitive_tc( org.omg.CORBA.TCKind tcKind )
    {
        return new com.engroom.CORBA.TypeCode( tcKind ) ;
    }

    public org.omg.CORBA.portable.OutputStream create_output_stream()
    {
        OpStream o = new OpStream() ;

        o.setORB( this ) ;

        return o ;
    }

    public void connect( org.omg.CORBA.Object obj )
    {
        try
        {
            ( ( org.omg.CORBA.DynamicImplementation )obj )._get_delegate() ;
        }
        catch ( org.omg.CORBA.BAD_OPERATION e )
        {
            ERObject ero = ERObject.createDelegate( this, obj ) ;

            getBOA().add( ero ) ;

            ero.getIOR() ;

            if ( firstConnect )
            {
                new Thread( boa ).start() ;
                firstConnect = false ;
            }
        }
    }

    public boolean getUseIPno() { return useipno ; }

    public void setUseIPno( boolean u ) { useipno = u ; }

    public String getInitialObjectKey() { return initialObjectKey ; }

    public void setInitialObjectKey( String key ) { initialObjectKey = key ; }

    public void disconnect( org.omg.CORBA.Object obj )
    {
        ERObject ero ;

        if ( obj instanceof ERObject )
        {
            ero = ( ERObject )obj ;
        }
        else if ( obj instanceof org.omg.CORBA.portable.ObjectImpl )
        {
            ero = ( ERObject )( ( org.omg.CORBA.portable.ObjectImpl )obj )._get_delegate() ;
        }
        else
        {
            throw new org.omg.CORBA.UNKNOWN( "Unknown object type" ) ;
        }

        ero.disconnect() ;
    }

    // additional static methods for ORB initialization

    private void set_parameters( java.util.Properties props )
    {
        java.util.Enumeration e = props.keys() ;

        while ( e.hasMoreElements() )
        {
            String key = ( String )e.nextElement() ;
            String val = props.getProperty( key ) ;
            char type ;

            try
            {
                if ( key.equals( "ORBport" ) )
                {
                    port =  Integer.parseInt( val ) ;
                }
                else if ( key.equals( "ORBlog" ) )
                {
                    Logger.log( new Boolean( val ).booleanValue() ) ;
                }
                else if ( key.equals( "ORBuseipno" ) )
                {
                    setUseIPno( new Boolean( val ).booleanValue() ) ;
                }
                else if ( key.equals( "ORBInitialObjectKey" ) )
                {
                    setInitialObjectKey( val ) ;
                }
                else if ( key.equals( "ORBtimeout" ) )
                {
                    timeout = Integer.parseInt( val ) ;
                }
                else if ( key.equals( "ORBidle" ) )
                {
                    idle = Integer.parseInt( val ) ;
                }
                else if ( key.equals( "org.omg.CORBA.ORBInitialPort" ) ||
                          key.equals( "ORBInitialPort" ) )
                {
                    initialPort = Integer.parseInt( val ) ;
                }
                else if ( key.equals( "org.omg.CORBA.ORBInitialHost" ) ||
                          key.equals( "ORBInitialHost" ) )
                {
                    initialHost = val ;
                }
                else if ( key.equals( "ORBhost" ) )
                {
                    host = val ;
                }
            }

            catch( NumberFormatException n )
            {
                throw new org.omg.CORBA.BAD_PARAM( "bad integer format " + val ) ;
            }
        }
    }

    private void set_parameters( java.applet.Applet app, String[] args, java.util.Properties props )
    {
        if ( props != null )
            set_parameters( props ) ;

        props = new java.util.Properties() ;

        String val ;
        String key ;

        for ( int i = 0 ; args != null && i < args.length ; i++ )
        {
            key = args[ i ] ;

            if ( !key.startsWith( "-ORB" ) )
                continue ;

            key = key.substring( 1 ) ;

            int index = key.indexOf( '=' ) ;

            if ( index == -1 )
                val = "true" ;
            else
            {
                val = key.substring( index + 1 ) ;
                key = key.substring( 0, index ) ;
            }

            // System.out.println( key + "=" + val ) ;

            props.put( key, val ) ;
        }

        set_parameters( props ) ;

        if ( app != null )
        {
            props = new java.util.Properties() ;

            for ( int j = 0 ; j < ORBparams.length ; j++ )
            {
                key = ORBparams[ j ] ;
                val = app.getParameter( key ) ;

                if ( val != null )
                    props.put( key, val ) ;
            }

            set_parameters( props ) ;
        }
    }

    public void set_parameters( String[] args, java.util.Properties props )
    {
        set_parameters( null, args, props ) ;
    }

    public void set_parameters( java.applet.Applet app, java.util.Properties props )
    {
        set_parameters( app, null, props ) ;
    }

    public static void die( String msg )
    {
        System.err.println( msg ) ;
        System.exit( -1 ) ;
    }

    public static void notimplemented( String msg )
    {
        die( msg + " not implemented!" ) ;
    }

/*
    private static int getUniquePort()
    {
        int port = -1 ;

        try
        {
            java.net.DatagramSocket ds = new java.net.DatagramSocket() ;

            port = ds.getLocalPort() ;

            ds.close() ;
        }
        catch ( java.net.SocketException e )
        {
            System.err.println( e.getMessage() ) ;
        }

        return port ;
    }
*/

    public synchronized java.net.ServerSocket getServerSocket()
        throws java.io.IOException
    {
        /* if ( port == -1 )
            port = getUniquePort() ; */

        java.net.ServerSocket sock = new java.net.ServerSocket( port, 15 ) ;

        // sock.setSoTimeout( getIdle() * 1000 ) ;

        Logger.println( "Using port " + port ) ;

        return sock ;
    }
}
